const io = require("socket.io");
const AuthService = require("./AuthService");
const UpService = require("./UpService");
const UpdaterSocket = require("./UpdaterSocket");
const MonitoringSocket = require("./MonitoringSocket");
const SocketRooms = require("./SocketRooms");
const SystemLogger = require("../logs/SystemLogger");
const MonitoringApacheCloud = require("./MonitoringApacheCloud");

class SocketService {
    getUpdaterSocket() {
        return UpdaterSocket;
    }
    getMonitoringSocket() {
        return MonitoringSocket;
    }
    getMonitoringApacheCloud() {
        return MonitoringApacheCloud;
    }
    async start(http) {
        try {
            this.io = io(http, { cors: "*" });

            UpdaterSocket.setSocketIo(this.io);
            MonitoringSocket.setSocketIo(this.io);
            MonitoringApacheCloud.setSocketIo(this.io);

            await MonitoringSocket.loadInitialParams();
            await MonitoringApacheCloud.loadMonitoringParams();
            this.io.on("connection", (socket) => {
                try {
                    const { log: info, agent, version, cdfilial, cdcaixa, ip, hostname } = this.getSocketHeaderInfo(socket);
                    const { page } = this.getQueryParams(socket);
                    if (agent) {
                        SystemLogger.log(`OnConnection (${info}): ${socket.id}`);
                    }
                    switch (agent) {
                        case "updater":
                            socket.CDFILIAL = cdfilial;
                            socket.CDCAIXA = cdcaixa || null;
                            UpdaterSocket.validateVersionCompatibility(socket, version, info, cdfilial, cdcaixa);
                            UpdaterSocket.validateOtherConnections(cdfilial, cdcaixa).then(_ => {
                                UpdaterSocket.joinDefaultRoom(socket);
                            }).catch(error => {
                                UpdaterSocket.updateInfoDuplicateConnection(cdfilial, cdcaixa, hostname, ip);
                                socket.disconnect();
                                SystemLogger.error(error);
                            });
                            break;
                        case "client":
                            socket.CDFILIAL = cdfilial;
                            MonitoringSocket.hasConsumersForMonitoring().then( hasConsumers => {
                                if(hasConsumers) {
                                    MonitoringSocket.startMonitoring([socket]);
                                    MonitoringSocket.connectedClient([cdfilial]);
                                }
                            });
                            MonitoringSocket.joinDefaultRoom(socket);
                            break;
                        default:
                            switch( page ) {
                                case 'monitoring':
                                    MonitoringSocket.joinRooms(socket);
                                    MonitoringSocket.connectedUser();
                                    MonitoringSocket.startMonitoring();
                                    break;
                                case 'updates':
                                    UpdaterSocket.joinRooms(socket);
                                    break;
                                case 'updatesPerCasher':
                                    UpdaterSocket.joinRooms(socket, true);
                                    break;
                                case 'parametrizacoes':
                                    socket.join(SocketRooms.master.parametrizacoesPage);
                                    break;
                                case 'monitoringCloud':
                                    socket.join(SocketRooms.master.monitoringCloudPage);
                                    MonitoringApacheCloud.startMonitoring();
                                    break;
                                case 'serviceManagement':
                                    socket.join(SocketRooms.master.serviceManagement)
                            }
                    }

                    /** Garante que o portal está conectado */

                    socket.on("login", (data) => {
                        if(!data) {
                            socket.disconnect();
                            return;
                        }
                        this.login(socket, data);
                    });

                    /**
                     * Exportação de vendas
                     */
                    socket.on("iniciaExportacao", (data) => {
                        try {
                            data.payload = JSON.parse(data.payload);
                            UpService.setInsereServidor(
                                data.CDFILIAL,
                                data.CDCAIXA,
                                data.CDLOJA,
                                data.payload
                            );
                        } catch (e) {}
                    });

                    socket.on("disconnect", async () => {
                        let { agent, cdfilial, cdcaixa } = this.getSocketHeaderInfo(socket);
                        agent = agent.toLowerCase();
                        if( agent === 'client') {
                            MonitoringSocket.connectedClient([cdfilial], 'N');
                        } else if ( agent === 'updater' ) {
                            const { CDFILIAL, CDCAIXA = null } = socket;
                            const params = {CDFILIAL};
                            if(CDCAIXA) params.CAIXAS = [CDCAIXA];
                            let socketUpdater = await UpdaterSocket.getUpdaterSocketByUnitAndCasher(cdfilial, cdcaixa);
                            UpdaterSocket.connectedUpdater([params], !!socketUpdater, !!socketUpdater);
                        } else { // master
                            MonitoringSocket.disconnectedUser();
                        }
                    });
                    MonitoringApacheCloud.registerEvents(socket, agent);
                    MonitoringSocket.registerEvents(socket, agent);
                    UpdaterSocket.registerEvents(socket, agent);
                } catch (error) {
                    SystemLogger.error(error);
                }
            });
        } catch (error) {
            SystemLogger.error(error);
        }
    }

    getQueryParams(socket) {
        const defaults = {
            page: '',
        };
        const { query } = socket.handshake;
        return ({
            ...defaults,
            ...query
        });
    }

    getSocketHeaderInfo(socket) {
        let { headers } = socket.handshake;
        const defaults = {
            agent: "",
            cdfilial: "",
            cdcaixa: "",
            hostname: "",
            version: "",
            "user-agent": "",
            "x-real-ip": socket.handshake.address,
        };
        let { agent, cdfilial, cdcaixa, hostname, version, ...others } = {
            ...defaults,
            ...headers,
        };
        let ip = others["x-real-ip"];
        let userAgent = others["user-agent"];
        let log = [agent, cdfilial, cdcaixa, hostname, ip]
            .filter((value) => value)
            .join(" | ");
        return { log, agent, cdfilial, cdcaixa, hostname, version, ip, userAgent };
    }

    login(socket, data) {
        // Não é uma filial que está conectando e sim um portal
        if (!data.CDFILIAL) {
            AuthService.autenticar(data.email, data.password)
                .then((result) => {
                    socket.email = data.email;
                    socket.emit("confirmLogin", {
                        error: false,
                        message: "CONNECTED_SUCCESS",
                        result: result,
                    });

                    if (!data.slave && !data.updater) {
                        MonitoringSocket.connectedClient();
                        UpdaterSocket.connectedUpdater();
                    }
                })
                .catch((error) => {
                    SystemLogger.error(error);
                    this.io
                        .to(socket.id)
                        .emit("message", "Você não é um usuário autorizado!");
                    socket.disconnect();
                });
        } else {
            // Uma filial se coneta ao master (Updater ou Monitoring)
            AuthService.autenticar(data.email, "", data.CDFILIAL)
                .then(() => {
                    socket.email = data.email;

                    // IF -> Conexão de client do Updater
                    // ELSE -> Conexão de client do Monitoring
                    if (data.updater) {
                        const {CDFILIAL, CDCAIXA = null} = socket;
                        const params = {CDFILIAL};
                        if(CDCAIXA) {
                            params.CAIXAS = [CDCAIXA];
                        }
                        UpdaterSocket.connectedUpdater(params);
                    } else {
                        MonitoringSocket.connectedClient([socket.CDFILIAL]);
                    }
                })
                .catch((error) => {
                    SystemLogger.error(error);
                    socket.emit("message", "Você não é um usuário autorizado!");
                    socket.disconnect();
                });
        }
    }

}

module.exports = new SocketService();