const updaterModel = require("../model/updaterModel");
const base64 = require("../functions/Base64");
const semver = require("semver");
const SocketRooms = require("./SocketRooms");
const MonitoringSocket = require('./MonitoringSocket');
const UpdaterService = require('./UpdaterService');
const SystemLogger = require("../logs/SystemLogger");
const UpdaterLogger = require("../logs/UpdaterLogger");

class UpdaterSocket {
    unitsWithIncompatibleVersion = {};
    unitsWithMultiConn = {};
    infoDuplicateConnection = {};
    setSocketIo(io) {
        this.io = io;
    }
    getSocketIo() {
        return this.io;
    }
    getUpdateRoomName(CDCAIXA = null) {
        const {
            updatesPage,
            updatesPerCasherPage
        } = SocketRooms.master;
        return CDCAIXA ? updatesPerCasherPage : updatesPage;
    }
    async getConnectedUnitsWithCasher(){
        const socketUpdaters = await this.getUpdaters();
        const socketInfo = socketUpdaters.reduce((acc, socket) => {
            acc[socket.CDFILIAL] = acc[socket.CDFILIAL] || new Set();
            if(socket.CDCAIXA) {
                acc[socket.CDFILIAL].add(socket.CDCAIXA);
            }
            return acc;
        }, {});
        return Object.keys(socketInfo).map( CDFILIAL => {
            const info = {CDFILIAL};
            if(socketInfo[CDFILIAL].size) info.CAIXAS = [...socketInfo[CDFILIAL]];
            return info;
        });
    }

    async connectedUpdater(UNITS = null, CONNECTED = true, WARNING = false) {
        UNITS = Array.isArray(UNITS) ? UNITS : await this.getConnectedUnitsWithCasher();
        let unitsWithoutCasher = [];
        let unitsWithCasher = [];
        UNITS.forEach(unit => {
            if(unit.CAIXAS) {
                unitsWithCasher.push(unit);
            } else {
                unitsWithoutCasher.push(unit);
            }
        })
        if(unitsWithoutCasher.length) {
            this.getRoom(SocketRooms.master.updatesPage)
            .to(SocketRooms.master.parametrizacoesPage).emit("updaterConnected", {UNITS: unitsWithoutCasher, CONNECTED, WARNING});
        }
        if(unitsWithCasher.length) {
            this.getRoom(SocketRooms.master.updatesPerCasherPage).emit("updaterConnected", {UNITS: unitsWithCasher, CONNECTED, WARNING});
        }
    }
    joinDefaultRoom( socket ) {
        socket.join(SocketRooms.updater.default);
    }
    joinRooms( socket, isPerCasher = false ) {
        const pages = [SocketRooms.master.default, this.getUpdateRoomName(isPerCasher)];
        socket.join(pages);
    }
    updateInfoDuplicateConnection(CDFILIAL, CDCAIXA, hostname, address, isActive = false) {
        this.infoDuplicateConnection[CDFILIAL] = this.infoDuplicateConnection[CDFILIAL] || {};
        let infoDuplicateConn = this.infoDuplicateConnection[CDFILIAL];
        if(CDCAIXA) {
            this.infoDuplicateConnection[CDFILIAL][CDCAIXA] = this.infoDuplicateConnection[CDFILIAL][CDCAIXA] || {};
            infoDuplicateConn = this.infoDuplicateConnection[CDFILIAL][CDCAIXA];
        }
        if(isActive) {
            infoDuplicateConn.ACTIVE = {
                hostname, address
            };
        } else {
            infoDuplicateConn[hostname] = address;
        }
    }
    addUnitWithMultipleConnection(CDFILIAL, CDCAIXA = null) {
        this.unitsWithMultiConn[CDFILIAL] = this.unitsWithMultiConn[CDFILIAL] || new Set();
        if(CDCAIXA) {
            this.unitsWithMultiConn[CDFILIAL].add(CDCAIXA);
        }
    }
    clearMultipleConnectionWarning(CDFILIAL, CDCAIXA = null) {
        if(CDCAIXA) {
            (this.unitsWithMultiConn[CDFILIAL] || new Set()).delete(CDCAIXA);
            delete this.infoDuplicateConnection[CDFILIAL][CDCAIXA];
        } else {
            delete this.unitsWithMultiConn[CDFILIAL];
            delete this.infoDuplicateConnection[CDFILIAL];
        }
    }
    async hasUnitUpdaterConnected(CDFILIAL, CDCAIXA = null) {
        const socketUpdater = await this.getUpdaterSocketByUnitAndCasher(CDFILIAL, CDCAIXA);
        return !!socketUpdater;
    }
    async validateOtherConnections(CDFILIAL, CDCAIXA = null) {
        const socketUpdater = await this.getUpdaterSocketByUnitAndCasher(CDFILIAL, CDCAIXA);
        const unitConnected = !!socketUpdater;
        const resultInfo = { CDFILIAL };
        if(unitConnected) {
            const { headers = {}, address} = socketUpdater.handshake;
            const { hostname } = headers;
            this.updateInfoDuplicateConnection(CDFILIAL, CDCAIXA, hostname, address, true);
            this.addUnitWithMultipleConnection(CDFILIAL, CDCAIXA);
            if(CDCAIXA) {
                resultInfo.CAIXAS = [CDCAIXA];
            }
            const room = this.getUpdateRoomName(CDCAIXA);
            this.getRoom(room).emit('updaterWarning', {UNITS: [resultInfo], WARNING: true});
            let casher = CDCAIXA ? `_${CDCAIXA}` : '';
            let errorMessage = `UNIT_${CDFILIAL}${casher}_IS_ALREADY_CONNECTED`;
            return Promise.reject(new Error(errorMessage));
        }
        return Promise.resolve();
    }
    notifyMasterUpdatersWithInvalidVersion(socket = null, UNITS = null) {
        const socketOrRoomToEmit = socket || this.getRoom(SocketRooms.master.updatesPage);
        const response = UNITS || this.getUnitsWithIncompatibleVersion();
        socketOrRoomToEmit.emit('updaterWithInvalidVersion', response);
    }
    getUnitsWithIncompatibleVersion() {
        return Object.keys(this.unitsWithIncompatibleVersion).map( CDFILIAL => {
            const item = {CDFILIAL};
            const CAIXAS = this.unitsWithIncompatibleVersion[CDFILIAL];
            if(CAIXAS.size) {
                item.CAIXAS = [...CAIXAS];
            }
            return item;
        });
    }
    getUnitsWithWarning(onlyUnitCode = false){
        const allUnits = Object.keys(this.unitsWithIncompatibleVersion).concat(
            Object.keys(this.unitsWithMultiConn)
        );
        const uniqueUnits = [... new Set(allUnits)];
        if(onlyUnitCode) return uniqueUnits;
        const response = uniqueUnits.reduce( (acc, CDFILIAL) => {
            const casherWithVersionProblem = this.unitsWithIncompatibleVersion[CDFILIAL] || new Set();
            const casherWithConnProblem = this.unitsWithMultiConn[CDFILIAL] || new Set();
            const casherWithWarning = new Set([...casherWithVersionProblem].concat([...casherWithConnProblem]));
            acc[CDFILIAL] = [...casherWithWarning];
            return acc;
        }, {});
        return response;
    }
    validateVersionCompatibility(socket, version, info, CDFILIAL, CDCAIXA) {
        const { MIN_UPDATER_VERSION_SUPPORTED = 'any' } = process.env;
        if (MIN_UPDATER_VERSION_SUPPORTED !== 'any' && semver.gt(MIN_UPDATER_VERSION_SUPPORTED, version)) {
            socket.emit("invalidVersion", {
                supportedUpdaterVersion: MIN_UPDATER_VERSION_SUPPORTED,
                updaterVersion: version,
            });

            SystemLogger.error(
                `Disconnecting: (${info}): ${socket.id} - Versão do updater inferior a ${MIN_UPDATER_VERSION_SUPPORTED}`
            );
            this.unitsWithIncompatibleVersion[CDFILIAL] = this.unitsWithIncompatibleVersion[CDFILIAL] || new Set();
            const response = {CDFILIAL};
            if(CDCAIXA) {
                this.unitsWithIncompatibleVersion[CDFILIAL].add(CDCAIXA);
                response.CAIXAS = [CDCAIXA];
            }
            this.notifyMasterUpdatersWithInvalidVersion(null, [response]);
            socket.disconnect();
            throw new Error("UPDATER_VERSION_INCOMPATIBLE");
        } else {
            if(CDCAIXA) {
                (this.unitsWithIncompatibleVersion[CDFILIAL] || new Set()).delete(CDCAIXA);
            } else {
                delete this.unitsWithIncompatibleVersion[CDFILIAL];
            }
            socket.emit("validVersion");
        }
    }
    getRoom(roomName) {
        return this.getSocketIo().to(roomName);
    }
    getDownloadLogRoom(CDFILIAL, CDCAIXA, type) {
        const {
            downloadLogTxt,
            downloadLogZip
        } = SocketRooms.master;
        const roomFunction = type === 'txt' ? downloadLogTxt : downloadLogZip;
        const downloadLogRoom = roomFunction.apply(this, [CDFILIAL, CDCAIXA]);
        return downloadLogRoom;
    }
    async getUpdaters() {
        return await this.getRoom(SocketRooms.updater.default).fetchSockets();
    }
    async getMasterFrontendClients(id = null){
        const sockets = await this.getRoom(SocketRooms.master.default).fetchSockets();
        return id ? sockets.find( sckt => sckt.id === id) : sockets;
    }
    async getUpdaterSocketByUnitAndCasher(CDFILIAL, CDCAIXA = null) {
        const socketUpdaters = await this.getUpdaters();
        const matchSocket = (socket) => {
            return socket.CDFILIAL == CDFILIAL && (!CDCAIXA || socket.CDCAIXA === CDCAIXA);
        };
        const updaterUnitSocket = socketUpdaters.find( matchSocket );
        return updaterUnitSocket;
    }
    registerUpdaterSocketEvents(socket) {
        socket.on("updateConfigResult", (resultConfigIni) => {

            this.getRoom(SocketRooms.master.parametrizacoesPage).emit(
                "updateConfigIniFinish",
                resultConfigIni
            );
        });

        socket.on("updaterLogResponse", async (logs) => {
            if(!logs.CDFILIAL) return;
            const {
                masterSocketId, 
                usingLatestLines = false, 
                ...responseLogs 
            } = logs;
            const roomLogsUnit = this.getRoom(SocketRooms.updater.log(socket.CDFILIAL, socket.CDCAIXA));
            if(usingLatestLines) {
                const masterSockets = await this.getMasterFrontendClients();
                const targetSocket = masterSockets.find( socket => socket.id === masterSocketId);
                if(targetSocket) targetSocket.emit('updaterLog', responseLogs);
            } else {
                roomLogsUnit.emit('updaterLog', responseLogs);
            }
        });

        socket.on("updaterResultInProgress", async (result) => {
            const room = this.getUpdateRoomName(socket.CDCAIXA);
            this.getRoom(room).emit("updaterInProgress", result);

            switch (result.action) {
                case 'install':
                    result.payload.OPERATION = 'INSTALACAO_EM_CURSO';
                break;
                case 'uninstall':
                    result.payload.OPERATION = 'REMOCAO_EM_CURSO';
                break;
            }
            await UpdaterService.updateEventTable(result.payload);
        });

        socket.on("updaterResultFinish", async (result) => {
            const room = this.getUpdateRoomName(socket.CDCAIXA);
            this.getRoom(room).emit("updaterFinish", result);
        });

        socket.on("updaterChangeServices", (result) => {
            const { CDFILIAL } = socket;
            switch(result.operation.toUpperCase()) {
                case 'INSTALL':
                    MonitoringSocket.addServiceToMonitoring(CDFILIAL, result.services);
                    UpdaterService.handleChangeServiceByUnit(CDFILIAL, result.services, 'INSTALL');
                    break;
                case 'REMOVE':
                    MonitoringSocket.removeServiceToMonitoring(CDFILIAL, result.services);
                    UpdaterService.handleChangeServiceByUnit(CDFILIAL, result.services, 'REMOVE');
                    break;
                default:
                    SystemLogger.error(`OPERATION_NOT_MAPPED ${result.operation}`);
            }
        })
        socket.on("testConnectionResult", (resultTestConn) => {
            this.getRoom(SocketRooms.master.parametrizacoesPage).emit(
                "testConnectionFinish",
                resultTestConn
            );
        });

        socket.on('downloadLogError', ({ error, message, param }) => {
            const { CDFILIAL, CDCAIXA, type } = param;
            const downloadLogRoomName = this.getDownloadLogRoom(CDFILIAL, CDCAIXA, type);
            const room = this.getRoom(downloadLogRoomName);
            room && room.emit('downloadLogResult', { error, message, param });
        });

        socket.on('serviceManagementResult', (param) => {
            const updaterServiceManagementRoom = this.getRoom(SocketRooms.master.serviceManagement);
            if (updaterServiceManagementRoom) {
                updaterServiceManagementRoom.emit('serviceManagementResult', param);
            }
        });
    }

    registerMasterSocketEvents(socket) {
        socket.on("updateConfigInit", async (params) => {
            const {
                CDFILIAL,
                CONFIG
            } = params;
            const socketUpdater = await this.getUpdaterSocketByUnitAndCasher( CDFILIAL );
            if (socketUpdater) {
                try {
                    socketUpdater.emit("updateConfig", CONFIG);
                } catch (error) {
                    socket.emit("updateConfigIniFinish", { error: true, message: error.message });
                }
            } else {
                socket.emit(
                    "updateConfigIniFinish",
                    {
                        error: true,
                        message: "CONNECTION_ACTIVE_REQUIRED_UPDATE_CONFIG_FILE_UNIT",
                    }
                );
            }
        });

        socket.on("getUpdaterLog", async (params) => {
            const updaterUnitSocket = await this.getUpdaterSocketByUnitAndCasher(
                params.CDFILIAL, params.CDCAIXA
            );
            if (updaterUnitSocket) {
                socket.join(SocketRooms.updater.log(params.CDFILIAL, params.CDCAIXA));
                updaterUnitSocket.emit("getUpdaterLog", {...params, masterSocketId: socket.id});
            }
        });

        socket.on('stopUpdaterLog', async (params) => {
            if(!params.CDFILIAL) return;
            const roomName = SocketRooms.updater.log(params.CDFILIAL, params.CDCAIXA);
            socket.leave(roomName);
            const roomLogs = this.getRoom(roomName);
            const connectedSockets = await roomLogs.fetchSockets();
            if(!connectedSockets.length) {
                const updaterSocket = await this.getUpdaterSocketByUnitAndCasher(params.CDFILIAL);
                if(updaterSocket) updaterSocket.emit('unwatchLogFile', params);
            }
        });

        socket.on('leaveDownloadLogRoom', async(params) => {
            const { CDFILIAL, CDCAIXA, type } = params;
            const roomName = this.getDownloadLogRoom(CDFILIAL, CDCAIXA, type);
            socket.leave(roomName);
        });

        socket.on("execUpdater", async (params) => {
            const updaterSocketUnit = await this.getUpdaterSocketByUnitAndCasher(params.CDFILIAL, params.CDCAIXA);
            if (updaterSocketUnit) {
                // Envia requisição de atualização para a loja          
                updaterSocketUnit.emit("updater", params);
            } else {
                SystemLogger.error(`SOCKET_UPDATER_NOT_FOUND_${params.CDFILIAL}`);
            }
        });

        socket.on('execUpdaterMultiInstall', async (params) => {
            const socketUpdaters = await this.getUpdaters();
            const unitsInfo = params.UNITS.reduce((acc, unitInfo) => {
                const {CDFILIAL, ...info} = unitInfo;
                acc[CDFILIAL] = info;
                return acc;
            }, {});
            socketUpdaters.forEach(socket => {
                const isTargetSocket = (socket.CDFILIAL in unitsInfo) && !socket.CDCAIXA;
                if(isTargetSocket) {
                    const { PRODFILIID, CURRENTVERSION } = unitsInfo[socket.CDFILIAL];
                    socket.emit('updater', {
                        ...params, 
                        PRODFILIID, 
                        CDFILIAL: socket.CDFILIAL,
                        CDPRODUCTID: params.CDPRODUCTID.toString(),
                        DEVOPSID: params.DEVOPSID.toString(),
                        CURRENTVERSION: CURRENTVERSION === '-' ? null : CURRENTVERSION
                    });
                }
            })
        });
        socket.on('testConnection', async (CDFILIAL) => {
            const socketUpdater = await this.getUpdaterSocketByUnitAndCasher( CDFILIAL );

            if(socketUpdater) {
                try {
                    const { rows } = await updaterModel.getConfigIniByUnit({P_CDFILIAL: CDFILIAL});

                    if (rows.length === 0) {
                        throw new Error(`Não foi encontrado o registro do arquivo de configuração para a filial ${CDFILIAL}`);
                    }
                    const configIniDecode = base64.decode(rows[0].CONFIG);
                    
                    socketUpdater.emit("updaterTestConnection", configIniDecode);
                } catch(error) {
                    socket.emit("testConnectionFinish", { error: true, message: error.message });
                }
            }
            else {
                socket.emit("testConnectionFinish", { error: true, message: "Falha no teste de conexão. Não existe conexão ativa com a loja no momento." });
            }
        });

        socket.on('removeTempWarning', async({CDFILIAL, CDCAIXA = null}) => {
            let room = this.getUpdateRoomName(CDCAIXA);
            const UNITS = CDCAIXA ? [{CDFILIAL, CAIXAS: [CDCAIXA]}] : [{CDFILIAL}];
            this.clearMultipleConnectionWarning(CDFILIAL, CDCAIXA);
            this.getRoom(room).emit('updaterWarning', {UNITS, WARNING: false});
        });
        
        socket.on('detailStatusConnection', async({CDFILIAL, CDCAIXA = null}) => {
            let info = this.infoDuplicateConnection[CDFILIAL];
            if(CDCAIXA) {
                info = this.infoDuplicateConnection[CDFILIAL][CDCAIXA];
            }
            socket.emit('detailStatusConnectionResult', info || {});
        });

        socket.on('downloadLog', async (param) => {
            try {
                const {
                    CDFILIAL,
                    CDCAIXA = null,
                    type
                } = param;
                const {
                    UPDATER_LOG_SERVER
                } = process.env;
                const updaterSocket = await this.getUpdaterSocketByUnitAndCasher(CDFILIAL, CDCAIXA);
                const downloadLogRoom = this.getDownloadLogRoom(CDFILIAL, CDCAIXA, type);
                const { fileName, useLocalLog } = await UpdaterLogger.getLocalLogInfo(CDFILIAL, CDCAIXA);
                if( useLocalLog ) {
                    param.downloadUrl = `${UPDATER_LOG_SERVER}/${fileName}`;
                    socket.emit('downloadLogResult', {error: false, param});
                    return;
                }
                if(updaterSocket) {
                    socket.join(downloadLogRoom);
                    updaterSocket.emit('downloadLog', param);
                }
            } catch (error) {
                SystemLogger.error(error);
            }
        });
        
        socket.on("serviceManagement", async (params) => {
            const socketUpdater = await this.getUpdaterSocketByUnitAndCasher(params.cdFilial, params.cdCaixa);
            if (socketUpdater) {
                socketUpdater.emit("serviceManagement", params);
            } else {
                socket.emit("serviceManagementResult", {
                    error: true, 
                    message: 'CONNECTION_ACTIVE_REQUIRED_UNIT'
                });
            }
        });
    }

    registerEvents(socket, agent) {
        if(agent === 'updater') {
            this.registerUpdaterSocketEvents(socket);
        } else if(agent !== 'client') {
            this.registerMasterSocketEvents(socket);
        }
    }
}

module.exports = new UpdaterSocket();