import { ModalLog } from '@/components/modal-log/common/modal-log';
import AlertHelper from '@/helpers/AlertHelper';
import { Component, Modal, ModalService, Button } from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { Socket } from 'socket.io-client';

class ModalLogController {
  private modalContent!: any;

  private modal!: Modal;

  private title!: string;

  private initialText!: string;

  private footer!: string;

  private row!: any;

  private socket?: Socket | null;

  private params!: {};

  constructor() {
    Metadata.get('modal-log', true).then((modalObj: any) => {
      this.modalContent = modalObj;
    });
  }

  public async create({ title, footer, events }: any): Promise<Modal> {
    this.modal?.destroy();
    this.title = title || 'Log';
    this.footer = footer || '';
    this.modal = ModalService.create(this.modalContent);
    this.setFooter(this.footer);
    this.setTitle(this.title);
    const form = this.getChildElement('modal-form');
    const closeButton = this.getCloseButton();
    const eventButtonLog = this.getButtonDownloadLog();
    if (closeButton) {
      this.bindCloseEvent(closeButton);
    }

    if (eventButtonLog) this.bindLogDownloadEvent(eventButtonLog);
    this.bindFormEvents(form, events);
    return this.modal;
  }

  private bindFormEvents(form: any, events: any) {
    if (!form || !events) return null;
    form.events = events;
  }

  private getHeader() {
    const header = this.getChildElement('modal-header');
    return header;
  }

  private getCloseButton() {
    const header = this.getHeader();
    if (!header) return null;
    const closeButton = header.rightSlot.find((child: any) => child.name === 'closeModalButton');
    return closeButton;
  }

  private getChildElement(name: string) {
    const element = this.modal?.children.find((child: any) => child.name === name);
    return element;
  }

  private bindCloseEvent(closeButton: any) {
    closeButton.events = {
      click: this.close.bind(this),
    };
  }

  public getRow() {
    return this.row;
  }

  public show(text?: string, row?: any) {
    this.initialText = text || this.initialText;
    this.row = row || this.row || {};
    this.setText(this.initialText);
    this.modal?.show();
    this.scrollToEnd();
    return this;
  }

  public setFooter(text: string) {
    const footer: any = this.getFooter();

    if (!footer || !footer.leftSlot) return this;
    const footerTextComponent = footer.leftSlot.find((child: any) => child.name === 'modal-footer-text');

    if (!footerTextComponent) return this;
    footerTextComponent.text = text;

    return this;
  }

  public setTitle(title: string) {
    const header = this.getHeader();
    if (!this.modal || !header) return this;
    const titleComponent = header.leftSlot.find((child: any) => child.name === 'titleText');
    if (titleComponent) {
      titleComponent.text = title;
    }
    return this;
  }

  private scrollToEnd() {
    const textarea = document.getElementById('log-textarea');
    if (textarea) {
      textarea.scrollTop = textarea.scrollHeight;
    }
    return this;
  }

  public close() {
    this.modal?.hide();
    this.modal?.destroy();
  }

  public getLogForm() {
    let logForm = null;
    try {
      logForm = this.modal?.getChildInstance('modal-form');
    } catch (error) {
      logForm = this.getChildElement('modal-form');
    }
    return logForm;
  }

  public getFooter() {
    let footer = null;
    try {
      footer = this.modal?.getChildInstance('footer');
    } catch (error) {
      footer = this.getChildElement('footer');
    }
    return footer;
  }

  public getFooterText() {
    const footer = this.getFooter();
    if (footer instanceof Component) {
      const textComponent = footer.getChildInstance('modal-footer-text');
      return textComponent;
    }
    return null;
  }

  public addText(text: string, previousText: string = '', laterText: string = '\n') {
    const form = this.getLogForm();
    if (form instanceof ModalLog) {
      form.addText(text, previousText, laterText);
      this.scrollToEnd();
    }
    return this;
  }

  public setText(text: string) {
    const form = this.getLogForm();
    if (form instanceof ModalLog) {
      form.setText(text);
      this.scrollToEnd();
    }
    return this;
  }

  public getText() {
    const form = this.getLogForm();
    if (form instanceof ModalLog) {
      return form.getText();
    }
    return null;
  }

  public clearText() {
    const form = this.getLogForm();
    if (form instanceof ModalLog) {
      form.clearText();
    }
    return this;
  }

  public setSocket(socket: any) {
    if (!this.socket) this.socket = socket;
  }

  private getButtonDownloadLog() {
    const footer: any = this.getFooter();
    if (footer) {
      const button = footer.rightSlot.find((child: any) => child.name === 'button-download-modal-log');
      return button;
    }
    return null;
  }

  private bindLogDownloadEvent(eventButtonLog: Button) {
    eventButtonLog.events = {
      click: this.downloadLog.bind(this),
    };
  }

  public parametersUnitAndCasher(params: any) {
    if (params) this.params = params;
    return null;
  }

  private dateDownloadLog(): string {
    const date = new Date();
    const year = date.getFullYear();
    const month = String(date.getMonth() + 1).padStart(2, '0');
    const day = String(date.getDay()).padStart(2, '0');
    const dateFormat = `${year}/${month}/${day}`;

    return dateFormat;
  }

  public downloadLog() {
    const dateFormat = this.dateDownloadLog();

    this.socket?.emit('downloadLog', { ...this.params, type: 'txt' });

    this.socket?.on('downloadLogResult', async ({ error, message, param }) => {
      const { CDFILIAL, CDCAIXA } = param;
      this.socket?.emit('leaveDownloadLogRoom', param);
      if (error) {
        AlertHelper.showError(message);
      } else {
        const { data } = await Http.get('/updater/getLogFile');

        const blob = new Blob([data.file]);
        const blobUrl = window.URL.createObjectURL(blob);
        const link = document.createElement('a');
        const casher = CDCAIXA ? `_cx_${CDCAIXA}` : '';

        link.href = blobUrl;
        link.download = `assistant_updater_log_${dateFormat}_filial_${CDFILIAL}${casher}`;
        link.click();
        AlertHelper.showSuccess('DOWNLOAD_IN_PROGRESS');
      }
      this.socket?.removeAllListeners('downloadLogResult');
    });
  }
}

export default new ModalLogController();
