import AlertHelper from '@/helpers/AlertHelper';
import { Form, IComponent, Modal, ModalService } from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';

export class ModalParametersController {
    private static modal: Modal | null;

    private static params: any;

    private static events: any = {};

    private static defaultEvents = {
        afterSave: () => {},
    };

    private static modalContent: any;

    private READONLY_FIELDS = ['CONFIG_FILE', 'SRV_PORT'];

    constructor() {
        if (!ModalParametersController.modalContent) {
            Metadata.get('updater-prod-config-modal', true).then((modalContent: any) => {
                ModalParametersController.modalContent = modalContent;
                ModalParametersController.modal = ModalService.create(modalContent);
            });
        }
    }

    private createModalInstance() {
        if (!ModalParametersController.modal) {
            ModalParametersController.modal = ModalService.create(ModalParametersController.modalContent);
        }
    }

    public setParams(params: any) {
        ModalParametersController.params = params;
    }

    public getParams(): any {
        return ModalParametersController.params;
    }

    public setEvents(events: any) {
        if (typeof events !== 'object') return;
        ModalParametersController.events = {
            ...ModalParametersController.defaultEvents,
            ...events,
        };
    }

    public removeEvent(eventName: string) {
        delete ModalParametersController.events[eventName];
    }

    public getEvents(): any {
        return ModalParametersController.events;
    }

    public hide() {
        if (ModalParametersController.modal) {
            ModalService.delete(ModalParametersController.modal);
        }
        ModalParametersController.modal?.hide();
        ModalParametersController.modal?.destroy();
        ModalParametersController.modal = null;
    }

    private getForm(): IComponent | Form | null {
        let form = null;
        const formName = 'updater-prod-config-form';
        try {
            form = Metadata.getInstance<Form>(formName);
            return form;
        } catch (error) {
            form = ModalParametersController.modal?.children.find((child: any) => child.name === formName);
            return form || null;
        }
    }

    private getIterableComponent() {
        try {
            const form = this.getForm();
            let iterableComponent = null;
            if (!form) return null;
            if (form instanceof Form) {
                return form?.getChildInstance('IterablecomponentParent');
            }
            iterableComponent = form.children?.find((child: any) => child.name === 'IterablecomponentParent');
            return iterableComponent || null;
        } catch (error) {
            return null;
        }
    }

    private setIterableData(data: any) {
        const iterableComponent: any = this.getIterableComponent();
        if (iterableComponent) {
            iterableComponent.datasource.data = data;
        }
    }

    private isEmpty(item: any) {
        if (!item) return true;
        if (Array.isArray(item) && !item.length) return true;
        if (typeof item === 'object' && !Object.keys(item).length) return true;
        return false;
    }

    public async show({ NRVERSAO, CDPRODUCTID }: any, isBeforeInstall = false) {
        this.createModalInstance();
        const response = {
            error: false,
            hasOpenModal: false,
            productConfig: {},
        };
        try {
            this.setParams({ NRVERSAO, CDPRODUCTID });

            const { data: prodConfig } = await this.getProductConfig({ NRVERSAOATUAL: NRVERSAO, UPDATERPRODID: CDPRODUCTID });
            if (prodConfig && prodConfig.error && prodConfig.message) {
                throw new Error(prodConfig.message);
            }
            response.productConfig = prodConfig.data;
            if (prodConfig && prodConfig.message && isBeforeInstall) {
                AlertHelper.showAlert(prodConfig.message);
            } else if (this.isEmpty(prodConfig.data)) {
                AlertHelper.showAlert('Projeto/versão não possui parâmetros');
                return response;
            }
            const components = this.getComponentsByData(prodConfig);

            this.setIterableData(components);
            if (prodConfig.message || !isBeforeInstall) {
                ModalParametersController.modal?.show();
                response.hasOpenModal = true;
                return response;
            }
            return response;
        } catch (error) {
            response.error = true;
            AlertHelper.showError(error);
            return response;
        }
    }

    private dynamicInputsHandler([name, value]: any) {
        return ({
            id: `IterablecomponentParent-${name}`,
            value,
            label: name,
            readonly: this.READONLY_FIELDS.includes(name),
        });
    }

    private getComponentsByData(prodConfig: any) {
        const components = Object.entries(prodConfig.data).map(this.dynamicInputsHandler.bind(this));
        return components;
    }

    private async getProductConfig(params: any) {
        return Http.post('/updater/getProdConfig', params);
    }

    private executeEvents(eventName: string, params: any) {
        const functionToCall = ModalParametersController.events[eventName];
        if (typeof functionToCall !== 'function') return;
        functionToCall.apply(this, [params]);
    }

    public async saveProductConfig() {
        try {
            const form: any = this.getForm();
            if (form instanceof Form) {
                const isValid = form.validate();
                if (!isValid) return;
            }
            if (form) {
                const inputs = [...form.element.querySelectorAll('input')];
                const CONFIG = inputs.reduce((configObj, input) => {
                    const paramName = input.id.replace('IterablecomponentParent-', '');
                    configObj[paramName] = input.value;
                    return configObj;
                }, {});
                const params = { ...this.getParams(), CONFIG };
                const response = await Http.post('/updater/saveProdConfig', params);
                if (response.data.error) {
                    AlertHelper.showError(response.data.message);
                } else {
                    AlertHelper.showSuccess(response.data.message);
                }
                this.hide();
                this.executeEvents('afterSave', CONFIG);
            }
        } catch (error) {
            AlertHelper.showError('Ocorreu um erro ao tentar salvar os parâmetros. Por favor, tente novamente.');
            console.error(error);
        }
    }
}

export default new ModalParametersController();
