import { Modal, ModalService } from '@zeedhi/common';
import { IDictionary, Http, Metadata } from '@zeedhi/core';
import { TekGrid, TekMemoryDatasource } from '@zeedhi/teknisa-components-common';
import AlertHelper from '@/helpers/AlertHelper';
import { Socket } from 'socket.io-client';
import { SocketHelper } from '@/helpers/SocketHelper';
import DataSourceHelper from '@/helpers/DataSourceHelper';

export class MonitoringCloudController {
  private monitoringModal!: Modal;

  private monitoringParams: any = [];

  private row: any = {};

  private title: string = '';

  private socket?: Socket;

  public async gridMounted() {
    const grid = Metadata.getInstance<TekGrid>('monitoring-cloud-grid');
    this.socket = SocketHelper.createConnection({ page: 'monitoringCloud' });
    this.socketListener();
    try {
      grid.datasource.loading = true;
      this.monitoringModal = await ModalService.createFromJson('monitoringCloudModal', true);

      const config = await Http.get('/data/parametrizacoesApacheCloud');
      this.monitoringParams = config.data.data;
      grid.datasource.updateData(config.data.data);
      const lastFilter = localStorage.getItem('LAST_FILTER_MONITORING_CLOUD');

      if (lastFilter) {
        (grid.datasource as TekMemoryDatasource).dynamicFilter = JSON.parse(lastFilter);
      }

      if (!this.monitoringParams.length) {
        AlertHelper.showAlert('Nenhum monitoramento ativo do tipo "Apache - Cloud" encontrado.');
      }
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      grid.datasource.loading = false;
    }
  }

  private getMainGrid(): TekGrid | null {
    try {
      const grid = Metadata.getInstance<TekGrid>('monitoring-cloud-grid');
      return grid;
    } catch (error) {
      return null;
    }
  }

  private getAllData(grid: TekGrid) {
    const { allData } = grid.datasource as TekMemoryDatasource;
    return allData;
  }

  private socketListener() {
    this.socket?.on('apacheCloudResult', (response: any) => {
      const { ID, STATUS } = response;
      const grid: TekGrid | null = this.getMainGrid();
      if (grid) {
        const connectionColumn = grid.getColumn('STATUS');
        const allData = this.getAllData(grid);
        const updatedRows = DataSourceHelper.updateGridRow(grid, 'ID', ID, { STATUS }, allData);
        updatedRows?.forEach((row: any) => {
          grid.reapplyConditions(row, [connectionColumn]);
        });
      }
    });
    this.socket?.on('stopMonitoring', (idsToStop: any) => {
      const grid: TekGrid | null = this.getMainGrid();
      if (grid) {
        const allData = this.getAllData(grid);
        const idxToRemove: Array<number> = [];
        allData.forEach((row: any, idx: number) => {
          if (idsToStop.includes(row.ID)) {
            idxToRemove.push(idx);
          }
        });
        idxToRemove.forEach((idx: number) => {
          allData.splice(idx, 1);
        });
        grid.datasource.updateData(allData.sort(this.orderGridData.bind(this)));
      }
    });
    this.socket?.on('startMonitoring', (monitoringConfig) => {
      const grid: TekGrid | null = this.getMainGrid();
      if (grid) {
        const allData = this.getAllData(grid);
        const hasMonitoring = allData.find((row: any) => row.ID === monitoringConfig.ID);
        if (!hasMonitoring) {
          allData.push(monitoringConfig);
          grid.datasource.updateData(allData.sort(this.orderGridData.bind(this)));
        }
      }
    });
    this.socket?.on('detailMonitoringApacheResult', (detail) => {
      const { datasource } = Metadata.getInstance<TekGrid>('monitoring-cloud-modal-grid');
      datasource.updateData(detail.sort(this.orderGridData.bind(this)));
    });
  }

  private orderGridData(element1: any, element2: any) {
    if (element1.STATUS === 'N' && element2.STATUS === 'S') {
      return -1;
    }
    if (element1.STATUS === 'S' && element2.STATUS === 'N') {
      return 1;
    }
    return element1.ID < element2.ID ? -1 : 1;
  }

  public async gridBeforeDestroy() {
    this.monitoringModal?.destroy();
    this.socket?.removeAllListeners();
    this.socket?.disconnect();
  }

  public async gridBeforeApplyFilter() {
    const { datasource } = Metadata.getInstance<TekGrid>('monitoring-cloud-grid');
    const filter = (datasource as TekMemoryDatasource).dynamicFilter;
    localStorage.setItem('LAST_FILTER_MONITORING_CLOUD', JSON.stringify(filter));
    ModalService.closeByName('monitoring-cloud-grid-filter-modal');
  }

  public getStatusColor({ row }: IDictionary<any>) {
    return row.STATUS === 'S' ? 'green' : 'red';
  }

  public buttonClickCloudMonitoring({ row, component }:any) {
    if (component.color === 'green') {
      AlertHelper.showSuccess('Tudo certo com esse monitoramento!');
    } else {
      this.row = row;
      const config = this.monitoringParams.find((element: any) => element.ID === row.ID) || {};
      this.title = `Monitoramento - ${config.DESCRICAO || ''}`;
      this.monitoringModal.show();
    }
  }

  public modalGridMounted() {
    this.socket?.emit('detailMonitoringApache', { ID: this.row.ID });
  }

  public setTitle({ component: grid }: any) {
    grid.title = this.title;
  }

  public closeModal() {
    this.monitoringModal.hide();
  }
}
