import { Http, Metadata } from '@zeedhi/core';
import { TekGrid, TekMemoryDatasource } from '@zeedhi/teknisa-components-common';
import { Button, DialogService, LoadingService, ModalService } from '@zeedhi/common';
import { SocketHelper } from '@/helpers/SocketHelper';
import AlertHelper from '@/helpers/AlertHelper';
import MessageHandler from '@/helpers/MessageHandler';
import { Socket } from 'socket.io-client';

type ServiceParameters = {
  readonly row: {
    CDFILIAL: string,
    MONITORAMENTO: string
  }
};

export class ServiceManagementController {
  private socket?: Socket;

  private async gridMounted() {
    const grid = this.tekGrid();
    this.socket = SocketHelper.createConnection({ page: 'serviceManagement' });
    this.socketListener();
    this.openFilter(grid);
  }

  private openFilter(grid: TekGrid): void {
    const filterButton = Metadata.getInstance<Button>(`${grid.name}_filterButton`);
    filterButton.click({} as Event, {} as HTMLElement);
  }

  private tekGrid(): TekGrid {
    const grid = Metadata.getInstance<TekGrid>('service-management-grid');
    return grid;
  }

  private beforeApplyFilter({ component, event }: any): void {
    event.preventDefault();
    const [cdFilialComponent] = component.getFilterInputs();
    const { value: CDFILIAL } = cdFilialComponent;
    this.updateGridData(CDFILIAL);
    ModalService.closeByName('service-management-grid-filter-modal');
    component.toolbarSlot[0].text = `Gerenciamento de Serviços - Filial ${CDFILIAL}`;
  }

  private async updateGridData(CDFILIAL: string): Promise<any> {
    try {
      const grid = this.tekGrid();
      LoadingService.show();

      const { data: response } = await Http.get('/updaterServiceManagement', { params: { CDFILIAL } });
      const datasource = (grid.datasource as TekMemoryDatasource);
      datasource.allData = response.data;
      datasource.data = response.data;
      datasource.total = response.data.length;
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      LoadingService.hide();
    }
  }

  public gridBeforeDestroy() {
    this.socket?.removeAllListeners();
    this.socket?.disconnect();
    ModalService.getModal('service-management-grid-filter-modal')?.destroy();
  }

  public restartService({ row }: ServiceParameters) {
    AlertHelper.showSuccess('PROCESS_REQUEST');
    const params = {
      cdFilial: row.CDFILIAL,
      serviceName: row.MONITORAMENTO,
      operation: 'RESTART',
    };
    this.socket?.emit('serviceManagement', params);
  }

  private socketListener(): void {
    this.socket?.on('serviceManagementResult', ({ error, message, param }) => {
      if (param && param.needUserConfirm && !param.userConfirmation) {
        return this.dialog(param);
      }
      if (error) {
        AlertHelper.showError(message, { NMSERVICE: param ? param.serviceName : '' });
      } else {
        AlertHelper.showSuccess(message);
      }
    });
  }

  private dialog(param: any): void {
    const { dependentServices } = param;
    const formatServiceName = (service: string) => `<strong>${service}</strong><br>`;
    const textServices = dependentServices.map(formatServiceName).join('');

    const message = MessageHandler.getMessageByCode('TITLE_CONFIRM_RESTART_DEPENDENT_SERVICES');
    const textDialog = `<p>${message}</p>\n${textServices}`;

    DialogService.show({
      name: 'confirm-restart-dialog',
      type: 'info',
      title: 'Reiniciar',
      text: `    
      ${textDialog} <br> ${MessageHandler.getMessageByCode('CONFIRM_RESTART_SERVICES')}`,
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          outline: true,
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              DialogService.hide();
              this.socket?.emit('serviceManagement', { ...param, userConfirmation: true });
            },
          },
        },
      ],
    });
  }
}
