import AlertHelper from '@/helpers/AlertHelper';
import { AlertService, Select, TextInput } from '@zeedhi/common';
import { IEventParam, Http, Metadata } from '@zeedhi/core';
import { TekCardTitle } from '@zeedhi/teknisa-components-vuetify';

export class UpVersionController {
  private regexVersion = /^\d{1,3}\.\d{1,3}\.\d{1,3}$/i;

  // eslint-disable-next-line
  private regexURL = /^(?:(?:(?:https?|ftp):)?\/\/)(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\\u00a1-\\uffff0-9]-*)*[a-z\\u00a1-\\uffff0-9]+)(?:\.(?:[a-z\\u00a1-\\uffff0-9]-*)*[a-z\\u00a1-\\uffff0-9]+)*(?:\.(?:[a-z\\u00a1-\\uffff]{2,})))(?::\d{2,5})?(?:[/?#]\S*)?$/i;

  public validateVersion({ component }: IEventParam<TextInput>): void {
    if (!this.regexVersion.test(component.value)) {
      AlertHelper.showError('Insira uma Versão válida. Ex: 1.2.003.');
    }
  }

  public validateURL({ component }: IEventParam<TextInput>): void {
    if (!this.regexURL.test(component.value)) {
      AlertHelper.showError('Insira uma URL válida.');
    }
  }

  public async saveNewVersion(): Promise<void> {
    try {
      const product = Metadata.getInstance<Select>('new-version-product');
      const numberVersion = Metadata.getInstance<TextInput>('new-version-number-version');
      const url = Metadata.getInstance<TextInput>('new-version-url');

      if (product.value === null) {
        throw new Error('Verifique se o produto foi selecionado.');
      }

      if (!this.regexVersion.test(numberVersion.value)) {
        throw new Error('Verifique se a versão esta preenchida da forma correta.');
      }

      if (!this.regexURL.test(url.value)) {
        throw new Error('Verifique se a url esta preenchida da forma correta.');
      }

      const replaceLastCharacter = numberVersion.value.replace(/\d$/, 'x');

      const params = {
        UPDATERPRODID: product.value,
        NRVERSAO: replaceLastCharacter,
        DSURL: url.value,
      };

      const { data } = await Http.post('/updater/installNewVersion', params);

      if (data && data.error) {
        throw new Error(data.message);
      }

      AlertHelper.showSuccess(data.message);
      product.value = '';
      numberVersion.value = '';
      url.value = '';
    } catch (error) {
      AlertHelper.showError(error);
    }
  }

  public changeProductId({ component, value }: any) {
    const versionTitle = Metadata.getInstance<TekCardTitle>('new-version-title');
    const data = component.datasource.data.find((item: any) => item.ID === value);
    if (data) versionTitle.title = `Nova Versão - ${data.NMPRODUCT}`;
  }

  get disableButtonSaveVersion(): boolean {
    const product = Metadata.getInstance<Select>('new-version-product');
    const numberVersion = Metadata.getInstance<TextInput>('new-version-number-version');
    const url = Metadata.getInstance<TextInput>('new-version-url');
    return product.value === null || numberVersion.value === '' || url.value === '';
  }

  private showAlert(message: any, colorStr: string): void {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 10000,
    });
  }
}
