import { AlertService, LoadingService, Modal, ModalService, Button, Select, DialogService, Text, Container } from '@zeedhi/common';
import { Config, Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';
import { SocketEvent } from '@/socket/socket-event';
import { CustomPagination } from '@/components/custom-pagination/common/custom-pagination';
import DataSourceHelper from '@/helpers/DataSourceHelper';
import AlertHelper from '@/helpers/AlertHelper';
import { ErrorHandler } from '@/helpers/ErrorHandler';
import ModalLogController from './ModalLogController';
import WarningController from './WarningController';
import ModalParameters from './ModalParametersController';
import ModalVersion from './ModalVersionController';
import VersionMinimumUpdater from '../helpers/VersionMinimumUpdater';

export class UpdatePerCasherController {
  public socket?: Socket;

  private fullscreenModal!: Modal;

  private logModal!: Modal;

  private defaultGridTitle!: String;

  private filterProductName!: String;

  private socketArgs = {
    email: 'assistant@teknisa.com',
    password: 'nineninenine',
    slave: false,
  };

  private getMainGrid(): TekGrid {
    const grid = Metadata.getInstance<TekGrid>('update-per-casher-grid');
    return grid;
  }

  public async createFullscreenModal() {
    this.fullscreenModal = await ModalService.createFromJson('update-per-casher-historico', true);
    this.logModal = await ModalService.createFromJson('updater-log-filial', true);
    const events = this.eventsModalLog();
    await ModalLogController.create({ events });
    await WarningController.create('updater-duplicate-conn-modal', {
      close: () => {
        const grid = this.getMainGrid();
        const { CDFILIAL, CDCAIXA } = grid.datasource.currentRow;
        DataSourceHelper.updateGridRow(grid, 'CDCAIXA', CDCAIXA, { WARNING: false });
        this.socket?.emit('removeTempWarning', { CDFILIAL, CDCAIXA });
        grid.buildGroupedData();
      },
    });
  }

  private verifyAndShowUnitStatus(row : any) {
    if (!row.CONNECTION_STATUS) AlertHelper.showError('CONNECTION_ACTIVE_REQUIRED_UNIT');
    return row.CONNECTION_STATUS;
  }

  private eventsModalLog() {
    const events = {
      onMounted: () => {
        this.socket?.removeAllListeners('updaterLog');
        this.socket?.on('updaterLog', (param) => {
          if (param.error === false) {
            ModalLogController.addText(param.message);
          }
        });
        this.socket?.emit('getUpdaterLog', ModalLogController.getRow());
      },
    };
    return events;
  }

  public async openLogBranchModal({ row }: any) {
    if (!this.verifyAndShowUnitStatus(row)) return;
    const params = { CDFILIAL: row.CDFILIAL, CDCAIXA: row.CDCAIXA };
    const footerText: string = `Log Updater - Filial: ${row.CDFILIAL} - Caixa: ${row.CDCAIXA}`;
    const events = this.eventsModalLog();
    await ModalLogController.create({ events });
    ModalLogController.setFooter(footerText).show('', params).parametersUnitAndCasher(params);
  }

  public onChangePageSize({ value }: any) {
    const grid = Metadata.getInstance<TekGrid>('update-per-casher-grid');
    const container = grid.getChildInstance<Container>('page-numbers');
    const customPagination = container.getChildInstance<CustomPagination>('custom-pagination');
    const allDataLength = (<any>grid.datasource).resultData.length;
    customPagination.page = 1;
    customPagination.length = Math.ceil(allDataLength / value);
    this.onChangePage({ page: 1 });
  }

  public calculatePageNumbers() {
    const grid = Metadata.getInstance<TekGrid>('update-per-casher-grid');
    const container = grid.getChildInstance<Container>('page-numbers');
    const { value: pageSize } = grid.getChildInstance<Select>('page-size').selectValue;
    const customPagination = container.getChildInstance<CustomPagination>('custom-pagination');
    const { search } = grid.datasource;
    const {
      filteredData,
      resultData,
    } = (<any>grid.datasource);
    const allDataSize = search ? filteredData.length : resultData.length;
    customPagination.length = Math.ceil(allDataSize / pageSize);
  }

  public onChangePage({ page = 1 }:any) {
    const grid = Metadata.getInstance<TekGrid>('update-per-casher-grid');
    const container = grid.getChildInstance<Container>('page-numbers');
    const customPagination = container.getChildInstance<CustomPagination>('custom-pagination');
    const pageInfo: Text = grid.getChildInstance('page-info');
    const pageSize: Select = grid.getChildInstance('page-size');
    const currentLimit = pageSize.value;
    let { resultData: allData } = (<any>grid.datasource);
    if (grid.datasource.search) {
      allData = (<any>grid.datasource).filteredData;
    }
    const { start, end, data } = this.getPage(allData, page, currentLimit);
    grid.datasource.data = data;
    pageInfo.text = `${start + 1} - ${end} of ${allData.length}`;
    customPagination.page = page;
    grid.buildGroupedData();
  }

  public openFullscreenModal({ row }: any) {
    LoadingService.show();

    const filter = {
      params: {
        CDFILIAL: row.CDFILIAL,
        CDCAIXA: row.CDCAIXA,
        CDPRODUCTID: row.CDPRODUCTID,
      },
    };

    Http.get('/updaterProdutoHistoricoPorCaixa', filter)
      .then((result) => {
        const grid = this.fullscreenModal.children[1];
        grid.datasource.data = result.data.data;
        this.fullscreenModal.show();
      })
      .catch((error) => {
        AlertHelper.showError(error);
      })
      .finally(() => {
        LoadingService.hide();
      });
  }

  public closeFullscreenModal() {
    this.fullscreenModal.hide();
  }

  public async searchInGrid(searchText: string) {
    const grid = Metadata.getInstance<TekGrid>('update-per-casher-grid');
    const { resultData } = (<any>grid.datasource);
    grid.datasource.search = searchText;
    (<any>grid.datasource).filteredData = resultData.filter((item: any) => {
      const matchCasher = item.CDCAIXA_NMCAIXA.toLowerCase().includes(searchText.toLowerCase());
      const matchState = (item.SGESTADO || '').toLowerCase().includes(searchText.toLowerCase());
      const checkEventStatus = item.IDSTEVENTO.toLowerCase() === 'instalacao_pendente' ? 'instalação pendente' : '';
      const matchEventStatus = checkEventStatus.includes(searchText.toLowerCase()) || (item.IDSTEVENTO || '').toLowerCase().includes(searchText.toLowerCase());
      const matchVersion = (item.NRVERSAOATUAL || '').includes(searchText);
      return matchCasher || matchVersion || matchState || matchEventStatus;
    });
    this.onChangePage({ page: 1 });
    this.calculatePageNumbers();
    return Promise.resolve((<any>grid.datasource).filteredData);
  }

  public async gridMounted(force: boolean = false) {
    if (!this.socket) {
      this.socket = io(Config.endPoint, {
        transports: ['websocket', 'polling'],
        query: {
          page: 'updatesPerCasher',
        },
      });
    }
    this.socket?.emit(SocketEvent.CONNECTION);
    this.socket?.emit('login', this.socketArgs);
    this.socketListener();
    const grid = Metadata.getInstance<TekGrid>('update-per-casher-grid');
    grid.setSearch = this.searchInGrid.bind(this);
    ModalLogController.setSocket(this.socket);
    const date = new Date().toLocaleDateString();
    const savedDate = localStorage.getItem('LAST_DATE_UPDATER');
    this.defaultGridTitle = grid.title;
    VersionMinimumUpdater.minimumSupportedVersionUpdater(grid);

    if (savedDate !== date || (typeof force === 'boolean' && force)) {
      LoadingService.show('Atualizando lista de sistemas...');
      const conexao = `${Config.metadataEndPoint}orgProducts`;
      Http.get(conexao)
        .then((result) => {
          if (result.data.isError) {
            ErrorHandler.getMessage({ message: 'ERROR_UPDATING_SYSTEM_LIST', params: { ERROR: result.data.response } });
          } else {
            AlertHelper.showSuccess('LIST_OF_SYSTEMS_UPDATE_SUCCESSFULLY');
          }
        })
        .catch((error) => {
          AlertHelper.showError(error);
        })
        .finally(() => {
          LoadingService.hide();
          localStorage.setItem('LAST_DATE_UPDATER', date);
          if (typeof force !== 'boolean') {
            this.openFilter(grid);
          }
        });
    } else {
        this.openFilter(grid);
    }
  }

  private openFilter(grid: TekGrid) {
    const filterButton = Metadata.getInstance<Button>(`${grid.name}_filterButton`);
    filterButton.click({} as Event, {} as HTMLElement);
  }

  public updateSystemList(): void {
    this.gridMounted(true);
  }

  private showAlert(message: any, colorStr: string) {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 10000,
    });
  }

  public executeDelete(row: any) {
    LoadingService.show();

    const params = {
      CDFILIAL: row.CDFILIAL,
      CDCAIXA: row.CDCAIXA,
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      DEVOPSID: row.CDPRODUCTID.toString(),
      PRODFILIID: row.PRODFILIID.toString(),
      CURRENTVERSION: row.NRVERSAOATUAL,
      INSTALLVERSION: null,
      CREATENEWVERSION: row.IDUTMULTVER,
      TYPE: 'REMOVE',
    };

    Http.post('/updater/removeVersion', params).then((result) => {
      if (result.data && result.data.error) {
        ErrorHandler.getMessage(result.data);
      } else {
        this.socket?.emit('execUpdater', params);
        AlertHelper.showSuccess('REMOVAL_REQUEST_SUCCESSFULLY');
      }
    })
      .catch((error) => {
        AlertHelper.showError(error);
      })
      .finally(() => {
        LoadingService.hide();
      });
  }

  public deleteVersion({ row }: any) {
    try {
      if (!this.verifyAndShowUnitStatus(row)) return;

      if (row.IDSTEVENTO === 'INSTALACAO_PENDENTE') {
        ErrorHandler.getMessage({ message: 'SYSTEM_NOT_INSTALLED' });
      }

      DialogService.show({
        name: 'remove-dialog',
        type: 'warning',
        title: 'Exclusão',
        text: 'Deseja realmente excluir essa versão desse sistema?',
        buttons: [
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            outline: true,
            events: {
              click: () => {
                DialogService.hide();
              },
            },
          },
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                DialogService.hide();
                this.executeDelete(row);
              },
            },
          },
        ],
      });
    } catch (error) {
      if (error instanceof Error) {
        this.showAlert(error.message, '#e68129');
      }
    }
  }

  private async executeInstall(row: any) {
    LoadingService.show();

    const params = {
      CDFILIAL: row.CDFILIAL,
      CDCAIXA: row.CDCAIXA,
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      DEVOPSID: row.CDPRODUCTID.toString(),
      PRODFILIID: row.PRODFILIID.toString(),
      CURRENTVERSION: row.NRVERSAOATUAL === '-' ? null : row.NRVERSAOATUAL,
      INSTALLVERSION: row.VERSION.NRVERSAO,
      CREATENEWVERSION: row.IDUTMULTVER,
      TYPE: 'INSTALL',
    };

    Http.post('/updater/installVersion', params)
      .then((result) => {
        if (result.data && result.data.error) {
          AlertHelper.showError(result.data.message);
        } else {
          this.socket?.emit('execUpdater', params);
          AlertHelper.showSuccess('REQUEST_INSTALLATION_UPDATE_SUCCESS');
        }
      })
      .catch((error) => {
        AlertHelper.showError(error);
      })
      .finally(() => {
        LoadingService.hide();
      });
  }

  public async installVersion({ row: paramRow }: any) {
    try {
      const availableVersion: any = this.getAvailableVersion(paramRow.PRODFILIID) || { value: {} };
      const VERSION = Object.assign(paramRow.VERSION || {}, availableVersion.value || {});
      const row = Object.assign(paramRow, { VERSION });
      if (!row.VERSION || Object.keys(row.VERSION).length === 0) {
        return AlertHelper.showAlert('Selecione a versão');
      }

      if (!this.verifyAndShowUnitStatus(row)) return;

      if (row.VERSION.IDDOWNLOADVER === 'N') {
        ErrorHandler.getMessage({ message: 'THIS_VERSION_NOT_AVAILABLE_DOWNLOAD' });
      }

      if (row.VERSION.IDDOWNLOADVER === 'E') {
        ErrorHandler.getMessage({ message: 'CLOUD_VERSION_DOWNLOAD_ERROR' });
      }
      const {
        CDFILIAL,
        CDCAIXA,
        CDPRODUCTID,
        VERSION: { NRVERSAO },
      } = row;

      const params = {
        CDFILIAL,
        CDCAIXA,
        UPDATERPRODID: CDPRODUCTID,
        NRVERSAOATUAL: NRVERSAO,
      };

      const checkVersionInstalled = await Http.post('/updater/checkVersion', params);

      if (checkVersionInstalled.data && checkVersionInstalled.data.error) {
        ErrorHandler.getMessage({ message: 'INSTALLED_VERSION' });
      }

      ModalParameters.setEvents({
        afterSave: (config: any) => {
          this.dialog(config, row);
        },
      });
      const { error, hasOpenModal, productConfig } = await ModalParameters.show({ CDPRODUCTID, NRVERSAO }, true);

      if (!error && !hasOpenModal) {
        this.dialog(productConfig, row);
      }
    } catch (error) {
      if (error instanceof Error) {
        AlertHelper.showError(error);
      }
    }
  }

  private isEmpty(item: any) {
    if (!item) return true;
    if (Array.isArray(item) && !item.length) return true;
    if (typeof item === 'object' && !Object.keys(item).length) return true;
    return false;
  }

  public async createProdConfigEditModal({ row }: any) {
    try {
      if (!row.VERSION) {
        return AlertHelper.showAlert('Selecione a versão');
      }
      const { CDPRODUCTID } = row;
      const { NRVERSAO } = row.VERSION;
      ModalParameters.removeEvent('afterSave');
      ModalParameters.show({ CDPRODUCTID, NRVERSAO });
    } catch (error) {
      AlertHelper.showError(error);
    }
  }

  private dialog(data: any, row?: any): void {
    let prodConfigValues = '';

    Object.entries(data).forEach(([key, value]) => {
      prodConfigValues += `<strong>${key}: ${value}</strong><br>`;
    });
    const hasParameters = !!prodConfigValues;
    const textDialogPrefix = hasParameters ? `<p>Parâmetros do Projeto</p>\n${prodConfigValues}` : '<strong>Projeto não possui parâmetros</strong>';
    DialogService.show({
      name: 'install-dialog',
      type: 'warning',
      title: 'Instalação',
      text: `    
      ${textDialogPrefix}
      <br>Deseja realmente instalar/atualizar esse sistema?                      
        `,
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          outline: true,
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              DialogService.hide();
              this.executeInstall(row);
            },
          },
        },
      ],
    });
  }

  private getAvailableVersion(id: any) {
    try {
      return Metadata.getInstance(`availableVersion_${id}`);
    } catch (error) {
      return null;
    }
  }

  private getFiltredUnit() {
    const grid = this.getMainGrid();
    const { resultData } = (<any>grid.datasource);
    const [FIRST_ROW] = resultData || [];
    const { CDFILIAL = null } = FIRST_ROW || {};
    return CDFILIAL;
  }

  private getCasherBySocketInfo(info: Array<any>) {
    const FILTRED_UNIT = this.getFiltredUnit();
    const UNIT_INFO = info.find((unit: any) => unit.CDFILIAL === FILTRED_UNIT);
    if (!UNIT_INFO || !UNIT_INFO.CAIXAS) return;
    const { CAIXAS } = UNIT_INFO;
    return CAIXAS;
  }

  private async socketListener() {
    this.socket?.on('updaterInProgress', (result) => {
      const messages: any = {
        INSTALACAO_EM_CURSO: 'Instalação iniciada...',
        ATUALIZACAO_EM_CURSO: 'Atualização iniciada...',
        REMOCAO_EM_CURSO: 'Remoção iniciada...',
      };
      const {
        PRODFILIID,
      } = result.payload;
      let messageCode = '';
      switch (result.operation) {
        case 'INSTALL':
          messageCode = 'INSTALACAO_EM_CURSO';
          break;
        case 'UPDATE':
          messageCode = 'ATUALIZACAO_EM_CURSO';
          break;
        case 'REMOVE':
          messageCode = 'REMOCAO_EM_CURSO';
          break;
        default:
          messageCode = 'INSTALACAO_EM_CURSO';
          break;
      }
      AlertHelper.showSuccess(messages[messageCode]);
      DataSourceHelper.updateGridRow(this.getMainGrid(), 'PRODFILIID', PRODFILIID, { IDSTEVENTO: messageCode });
    });

    this.socket?.on('updaterFinish', (result) => {
      const {
        PRODFILIID,
        INSTALLVERSION: NRVERSAOATUAL,
      } = result.payload;
      if (result.error) {
        AlertHelper.showError('ERROR_RUNNING_LAST_PROCESS_CHECK_HISTORY_INFORMATION');
        DataSourceHelper.updateGridRow(this.getMainGrid(), 'PRODFILIID', PRODFILIID, { IDSTEVENTO: null });
      } else {
        AlertHelper.showSuccess('OPERATION_COMPLETED_SUCCESSFULLY');
        const availableVersion = Metadata.getInstance<Select>(`availableVersion_${PRODFILIID}`);
        const IDSTEVENTO = result.operation === 'REMOVE' ? 'INSTALACAO_PENDENTE' : 'INSTALADO';
        availableVersion.setValue(null);
        DataSourceHelper.updateGridRow(this.getMainGrid(), 'PRODFILIID', PRODFILIID, { NRVERSAOATUAL, IDSTEVENTO });
      }
    });
    this.socket?.on('disconnect', () => {
      AlertHelper.showError('PAGE_WILL_RELOAD_TO_ESTABLISH_CONNECTION');
      setTimeout(() => {
        document.location.reload();
      }, 5000);
    });

    this.socket?.on('updaterConnected', async ({ UNITS, CONNECTED }) => {
      const CASHERS = this.getCasherBySocketInfo(UNITS);
      DataSourceHelper.updateGridRow(this.getMainGrid(), 'CDCAIXA', CASHERS, { CONNECTION_STATUS: CONNECTED });
    });

    this.socket?.on('updaterWarning', async ({ UNITS, WARNING = true }) => {
      const CASHERS = this.getCasherBySocketInfo(UNITS);
      DataSourceHelper.updateGridRow(this.getMainGrid(), 'CDCAIXA', CASHERS, { WARNING });
    });

    this.socket?.on('detailStatusConnectionResult', async (INFO) => {
      await WarningController.openUpdaterDuplicateConnModal(INFO);
    });
  }

  public setAvailableVersionFilter({ component }: any) {
    const grid = Metadata.getInstance<TekGrid>('update-per-casher-grid');
    const [firstRow] = grid.datasource.data;
    const { CDPRODUCTID } = firstRow;
    component.datasource.filter = { CDPRODUCTID };
  }

  public async beforeApplyFilter({ component, event }: any) {
    // Force to reload all available version column
    // component.datasource.updateData([]);
    event.preventDefault();
    const [productIdComponent] = component.getFilterInputs();
    const { value: PRODUCT_ID } = productIdComponent;
    this.updateGridData(PRODUCT_ID);
    ModalService.closeByName('updater-grid-filter-modal');
    component.toolbarSlot[0].text = `${this.defaultGridTitle} - ${this.filterProductName}`;
  }

  private getPage(dataset: any, page: number, limit: number) {
    const end: number = page * limit;
    const start = end - limit;
    const data = dataset.slice(end - limit, end);
    return {
      start,
      end: end > data.length ? data.length : end,
      data,
    };
  }

  public async updateGridData(PRODUCT_ID: any) {
    const grid = Metadata.getInstance<TekGrid>('update-per-casher-grid');
    const [{ datasource }]: any = grid.getFilterInputs('CDFILIAL');
    const { CDFILIAL } = datasource.currentRow;

    ModalService.closeByName('update-per-casher-grid-filter-modal');
    LoadingService.show();
    const result = await Http.get('/updaterProdutosFilialPerCasher', { params: { PRODUCT_ID, CDFILIAL } }).catch((err) => {
      AlertHelper.showError(err);
    }).finally(() => {
      LoadingService.hide();
    });

    (<any>grid.datasource).resultData = result.data.data;
    this.searchInGrid(grid.datasource.search);
    const { limit } = grid.datasource;
    const container: Container = grid.getChildInstance<Container>('page-numbers');
    const customPagination : CustomPagination = container.getChildInstance('custom-pagination');
    customPagination.length = Math.ceil(result.data.data.length / limit);
    this.onChangePage({ page: 1 });
  }

  public async changeProductId({ component, value }: any): Promise<void> {
    try {
      const selectGrid: any = Metadata.getInstance('update-per-casher-grid');
      const [{ datasource }]: any = selectGrid.getFilterInputs('CDFILIAL');
      datasource.updateData([]);

      const result = component.datasource.data.find((item: any) => item.ID === value);
      if (result && value) {
        datasource.loading = true;

        const { data } = await Http.get('/updaterProdutosPerCasher', { params: { PRODUCT_ID: value } });

        if (data && data.error) ErrorHandler.getMessage({ message: 'ERROR_GET_UNITS' });

        datasource.updateData(data.data);

        datasource.loading = false;
      }
    } catch (error) {
      AlertHelper.showError(error);
    }
  }

  public changeGetProductNameBranch({ component }: any): void {
    this.filterProductName = `Filial: ${component.searchValue}`;
  }

  public gridBeforeDestroy() {
    this.socket?.removeAllListeners();
    this.socket?.disconnect();
    this.fullscreenModal?.destroy();
    this.logModal?.destroy();
    WarningController.destroyModal('updater-duplicate-conn-modal');
    ModalService.getModal('update-per-casher-grid-filter-modal')?.destroy();
    ModalService.getModal('modal-log')?.destroy();
  }

  public getConnectionColor({ row }: any) {
    const color = row.CONNECTION_STATUS ? 'green' : 'red';
    return row.WARNING ? '#FFCD01' : color;
  }

  public moreInfoConnection({ row }: any) {
    WarningController.moreInfoConnection(row, this.socket);
  }

  public changeSelectVersion({ value }: IEventParam<Select>) {
    const grid = Metadata.getInstance<TekGrid>('update-per-casher-grid');
    grid.datasource.currentRow.VERSION = value;
  }

  public beforeOpenReport(reportParams: any) {
    const grid = Metadata.getInstance<TekGrid>('update-per-casher-grid');
    reportParams.dataSet = (<any>grid.datasource).resultData;
    reportParams.dataSet = JSON.parse(JSON.stringify(reportParams.dataSet));
    reportParams.metaData.filter = [{ label: 'Produto', value: this.filterProductName }];
    const { componentProps: { datasource, dataValue, dataText } } = grid.getColumn('IDSTEVENTO');
    const events = datasource.data;
    const messageEvents = events.reduce((acc: any, cv: any) => {
      acc[cv[dataValue]] = cv[dataText];
      return acc;
    }, {});
    reportParams.dataSet.forEach((item: any) => {
      item.IDSTEVENTO = messageEvents[item.IDSTEVENTO] || item.IDSTEVENTO;
    });
    return reportParams;
  }

  public async openModalVersion({ row }: any) {
    const title = `Filial: &nbsp${row.CDFILIAL} - ${row.NMFILIAL}<br>Caixa:\t${row.CDCAIXA} - ${row.NMCAIXA}`;
    ModalVersion.setParams(row);
    ModalVersion.setTitle(title);
    ModalVersion.show();
  }
}
