import { LoadingService, Modal, ModalService, Button, Select, DialogService, Text, Container } from '@zeedhi/common';
import { Config, Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';
import { SocketEvent } from '@/socket/socket-event';
import { CustomPagination } from '@/components/custom-pagination/common/custom-pagination';
import AlertHelper from '@/helpers/AlertHelper';
import DataSourceHelper from '@/helpers/DataSourceHelper';
import { ErrorHandler } from '@/helpers/ErrorHandler';
import { ModalMultipleInstallController } from './ModalMultipleInstallController';
import ModalLogController from './ModalLogController';
import WarningController from './WarningController';
import ModalParameters from './ModalParametersController';
import ModalVersion from './ModalVersionController';
import VersionMinimumUpdater from '../helpers/VersionMinimumUpdater';

export class UpdatesController {
  public socket?: Socket;

  private fullscreenModal!: Modal;

  private defaultGridTitle!: String;

  private filterRow!: any;

  private socketArgs = {
    email: 'assistant@teknisa.com',
    password: 'nineninenine',
    slave: false,
  };

  public async createFullscreenModal() {
    this.fullscreenModal = await ModalService.createFromJson('historico', true);
    await ModalMultipleInstallController.create();
    await WarningController.create('updater-duplicate-conn-modal', {
      close: () => {
        const grid = this.getMainGrid();
        const { CDFILIAL } = grid.datasource.currentRow;
        DataSourceHelper.updateGridRow(grid, 'CDFILIAL', CDFILIAL, { WARNING: false });
        this.socket?.emit('removeTempWarning', { CDFILIAL });
        grid.buildGroupedData();
      },
    });
    const events = this.eventsModalLog();
    await ModalLogController.create({ events });
  }

  private verifyAndShowUnitStatus(row : any) {
    if (!row.CONNECTION_STATUS) AlertHelper.showError('CONNECTION_ACTIVE_REQUIRED_UNIT');
    return row.CONNECTION_STATUS;
  }

  private eventsModalLog() {
    const events = {
      onMounted: () => {
        this.socket?.removeAllListeners('updaterLog');
        this.socket?.on('updaterLog', (param) => {
          if (param.error === false) {
            ModalLogController.addText(param.message);
          }
        });
        this.socket?.emit('getUpdaterLog', ModalLogController.getRow());
      },
    };
    return events;
  }

  public async openLogBranchModal({ row }: any) {
    if (!this.verifyAndShowUnitStatus(row)) return;
    const params = { CDFILIAL: row.CDFILIAL };
    const footer = `Log Updater - Filial: ${row.CDFILIAL}`;
    const events = this.eventsModalLog();
    await ModalLogController.create({ events });
    ModalLogController.setFooter(footer).show('', params).parametersUnitAndCasher(params);
  }

  private getMainGrid(): TekGrid {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    return grid;
  }

  public onChangePageSize({ value }: any) {
    const grid = this.getMainGrid();
    const container = grid.getChildInstance<Container>('page-numbers');
    const customPagination = container.getChildInstance<CustomPagination>('custom-pagination');
    const allDataLength = (<any>grid.datasource).resultData.length;
    customPagination.page = 1;
    customPagination.length = Math.ceil(allDataLength / value);
    this.onChangePage({ page: 1 });
  }

  public calculatePageNumbers() {
    const grid = this.getMainGrid();
    const container = grid.getChildInstance<Container>('page-numbers');
    const { value: pageSize } = grid.getChildInstance<Select>('page-size').selectValue;
    const customPagination = container.getChildInstance<CustomPagination>('custom-pagination');
    const { search } = grid.datasource;
    const {
      filteredData,
      resultData,
    } = (<any>grid.datasource);
    const allDataSize = search ? filteredData.length : resultData.length;
    customPagination.length = Math.ceil(allDataSize / pageSize);
  }

  public onChangePage({ page = 1 }:any) {
    const grid = this.getMainGrid();
    const container = grid.getChildInstance<Container>('page-numbers');
    const customPagination = container.getChildInstance<CustomPagination>('custom-pagination');
    const pageInfo: Text = grid.getChildInstance('page-info');
    const pageSize: Select = grid.getChildInstance('page-size');
    const currentLimit = pageSize.value;
    let { resultData: allData } = (<any>grid.datasource);
    if (grid.datasource.search) {
      allData = (<any>grid.datasource).filteredData;
    }
    const { start, end, data } = this.getPage(allData, page, currentLimit);
    grid.datasource.data = data;
    grid.selectedRows = [];
    pageInfo.text = `${start + 1} - ${end} of ${allData.length}`;
    customPagination.page = page;
    grid.buildGroupedData();
  }

  public openFullscreenModal({ row }: any) {
    LoadingService.show();

    const filter = {
      params: {
        CDFILIAL: row.CDFILIAL,
        CDPRODUCTID: row.ID,
      },
    };

    Http.get('/updaterProdutoHistorico', filter)
      .then((result) => {
        const grid = this.fullscreenModal.children[1];
        grid.datasource.data = result.data.data;
        this.fullscreenModal.show();
      })
      .catch((error) => AlertHelper.showError(error))
      .finally(() => {
        LoadingService.hide();
      });
  }

  public async searchInGrid(searchText: string | any) {
    const grid = this.getMainGrid();
    const { resultData } = (<any>grid.datasource);
    grid.datasource.search = searchText;
    (<any>grid.datasource).filteredData = resultData.filter((item: any) => {
      const matchUnit = item.FILIAL.toLowerCase().includes(searchText.toLowerCase());
      const matchState = (item.SGESTADO || '').toLowerCase().includes(searchText.toLowerCase());
      const matchStatus = (item.IDSTEVENTO || '').toLowerCase().includes(searchText.toLowerCase());
      const matchVersion = (item.NRVERSAOATUAL || '').includes(searchText);
      return matchUnit || matchState || matchVersion || matchStatus;
    });
    this.onChangePage({ page: 1 });
    this.calculatePageNumbers();
    return Promise.resolve((<any>grid.datasource).filteredData);
  }

  private addActionToolbar() {
    const grid = this.getMainGrid();

    const rightActions: any = grid.toolbarSlot.find((el: any) => el.tag === 'span' && el.component === 'ZdTag');
    const existsAction: boolean = rightActions && !!rightActions.children.find((el: any) => el.name === 'tooltip_multiple_install');
    if (rightActions && !existsAction) {
      rightActions.children.unshift(
        {
          name: 'tooltip_multiple_install',
          component: 'ZdTooltip',
          label: 'Instalação múltipla',
          bottom: true,
          children: [
            {
              name: 'multiple_install',
              component: 'ZdButton',
              flat: true,
              icon: true,
              iconName: 'mdi-download-multiple',
              events: {
                click: () => {
                  if (!grid.selectedRows.length) {
                    AlertHelper.showAlert('SELECT_UNITS_TO_INSTALL');
                    return;
                  }
                  const UNITS = grid.selectedRows.map((row: any) => row.CDFILIAL);
                  ModalMultipleInstallController.setParams({ ...this.filterRow, UNITS });
                  ModalMultipleInstallController.show();
                },
              },
            },
          ],
        },
      );
    }
  }

  public async gridMounted(force: boolean = false) {
    if (!this.socket) {
      this.socket = io(Config.endPoint, {
        transports: ['websocket', 'polling'],
        query: {
          page: 'updates',
        },
      });
    }
    ModalMultipleInstallController.setSocket(this.socket);
    ModalLogController.setSocket(this.socket);
    this.socket?.emit(SocketEvent.CONNECTION);
    this.socket?.emit('login', this.socketArgs);
    this.socketListener();
    const grid = this.getMainGrid();
    this.addActionToolbar();
    grid.setSearch = this.searchInGrid.bind(this);
    const date = new Date().toLocaleDateString();
    const savedDate = localStorage.getItem('LAST_DATE_UPDATER');
    this.defaultGridTitle = grid.title;
    VersionMinimumUpdater.minimumSupportedVersionUpdater(grid);

    if (savedDate !== date || (typeof force === 'boolean' && force)) {
      LoadingService.show('Atualizando lista de sistemas...');
      const conexao = `${Config.metadataEndPoint}orgProducts`;
      Http.get(conexao)
        .then((result) => {
          if (result.data.isError) {
            ErrorHandler.getMessage({ message: 'ERROR_UPDATING_SYSTEM_LIST', params: { ERROR: result.data.response } });
          } else {
            AlertHelper.showSuccess('LIST_OF_SYSTEMS_UPDATE_SUCCESSFULLY');
          }
        })
        .catch((error) => AlertHelper.showError(error))
        .finally(() => {
          LoadingService.hide();
          localStorage.setItem('LAST_DATE_UPDATER', date);
          if (typeof force !== 'boolean') {
            this.openFilter(grid);
          }
        });
    } else {
      this.openFilter(grid);
    }
  }

  private openFilter(grid: TekGrid) {
    const filterButton = Metadata.getInstance<Button>(`${grid.name}_filterButton`);
    filterButton.click({} as Event, {} as HTMLElement);
  }

  public updateSystemList(): void {
    this.gridMounted(true);
  }

  public executeDelete(row: any) {
    LoadingService.show();

    const params = {
      CDFILIAL: row.CDFILIAL,
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      PRODFILIID: row.PRODFILIID.toString(),
      DEVOPSID: row.ID.toString(),
      CURRENTVERSION: row.NRVERSAOATUAL,
      INSTALLVERSION: null,
      CREATENEWVERSION: row.IDUTMULTVER,
      TYPE: 'REMOVE',
    };

    Http.post('/updater/removeVersion', params).then((result) => {
      if (result.data && result.data.error) {
        ErrorHandler.getMessage(result.data);
      } else {
        this.socket?.emit('execUpdater', params);
        AlertHelper.showSuccess('REMOVAL_REQUEST_SUCCESSFULLY');
      }
    })
      .catch((error) => AlertHelper.showError(error))
      .finally(() => {
        LoadingService.hide();
      });
  }

  public deleteVersion({ row }: any) {
    try {
      if (!this.verifyAndShowUnitStatus(row)) return;

      if (row.IDSTEVENTO === 'INSTALACAO_PENDENTE') {
        ErrorHandler.getMessage({ message: 'SYSTEM_NOT_INSTALLED' });
      }

      DialogService.show({
        name: 'remove-dialog',
        type: 'warning',
        title: 'Exclusão',
        text: 'Deseja realmente excluir essa versão desse sistema?',
        buttons: [
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            outline: true,
            events: {
              click: () => {
                DialogService.hide();
              },
            },
          },
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                DialogService.hide();
                this.executeDelete(row);
              },
            },
          },
        ],
      });
    } catch (error) {
      if (error instanceof Error) {
        AlertHelper.showAlert(error.message);
      }
    }
  }

  private async executeInstall(row: any) {
    LoadingService.show();

    const params = {
      CDFILIAL: row.CDFILIAL,
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      PRODFILIID: row.PRODFILIID.toString(),
      DEVOPSID: row.ID.toString(),
      CURRENTVERSION: row.NRVERSAOATUAL === '-' ? null : row.NRVERSAOATUAL,
      INSTALLVERSION: row.VERSION.NRVERSAO,
      CREATENEWVERSION: row.IDUTMULTVER,
      TYPE: 'INSTALL',
    };

    Http.post('/updater/installVersion', params)
      .then((result) => {
        if (result.data && result.data.error) {
          ErrorHandler.getMessage(result.data);
        } else {
          this.socket?.emit('execUpdater', params);
          AlertHelper.showSuccess('REQUEST_INSTALLATION_UPDATE_SUCCESS');
        }
      })
      .catch((error) => AlertHelper.showError(error))
      .finally(() => {
        LoadingService.hide();
      });
  }

  private getAvailableVersion(id: any) {
    try {
      return Metadata.getInstance(`availableVersion_${id}`);
    } catch (error) {
      return null;
    }
  }

  public async installVersion({ row: paramRow, component }: any) {
    try {
      if (!this.verifyAndShowUnitStatus(paramRow)) return;
      const idToMatch = component.name.replace('ButtonDownload_', '');
      const availableVersion: any = this.getAvailableVersion(idToMatch) || { value: {} };
      const VERSION = Object.assign(paramRow.VERSION || {}, availableVersion.value || {});
      const row = Object.assign(paramRow, { VERSION });
      if (!row.VERSION || Object.keys(row.VERSION).length === 0) {
        return AlertHelper.showAlert('SELECT_VERSION');
      }

      if (row.VERSION.IDDOWNLOADVER === 'N') {
        ErrorHandler.getMessage({ message: 'THIS_VERSION_NOT_AVAILABLE_DOWNLOAD' });
      }

      if (row.VERSION.IDDOWNLOADVER === 'E') {
        ErrorHandler.getMessage({ message: 'CLOUD_VERSION_DOWNLOAD_ERROR' });
      }

      const {
        CDFILIAL,
        ID: CDPRODUCTID,
        VERSION: { NRVERSAO },
      } = row;

      const params = {
        CDFILIAL,
        UPDATERPRODID: CDPRODUCTID,
        NRVERSAOATUAL: NRVERSAO,
      };

      const checkVersionInstalled = await Http.post('/updater/checkVersion', params);

      if (checkVersionInstalled.data && checkVersionInstalled.data.error) {
        ErrorHandler.getMessage({ message: 'INSTALLED_VERSION' });
      }

      ModalParameters.setEvents({
        afterSave: (config: any) => {
          this.dialog(config, row);
        },
      });
      const { error, hasOpenModal, productConfig } = await ModalParameters.show({ CDPRODUCTID, NRVERSAO }, true);

      if (!error && !hasOpenModal) {
        this.dialog(productConfig, row);
      }
    } catch (error) {
      AlertHelper.showError(error);
    }
  }

  public async createProdConfigEditModal({ row: paramRow }: any) {
    try {
      const availableVersion: any = this.getAvailableVersion(paramRow.PRODFILIID) || { value: {} };
      const VERSION = Object.assign(paramRow.VERSION || {}, availableVersion.value || {});
      const row = Object.assign(paramRow, { VERSION });
      if (!row.VERSION || Object.keys(row.VERSION).length === 0) {
        return AlertHelper.showAlert('SELECT_VERSION');
      }
      const { NRVERSAO } = row.VERSION;
      const CDPRODUCTID = row.ID;

      ModalParameters.removeEvent('afterSave');
      ModalParameters.show({ CDPRODUCTID, NRVERSAO });
    } catch (error) {
      AlertHelper.showError(error);
    }
  }

  private dialog(data: any, row?: any): void {
    let prodConfigValues = '';

    Object.entries(data).forEach(([key, value]) => {
      prodConfigValues += `<strong>${key}: </strong>${value}<br>`;
    });
    const hasParameters = !!prodConfigValues;
    const textDialogPrefix = hasParameters ? `<p>Parâmetros do Projeto</p>\n${prodConfigValues}` : '<strong>Projeto não possui parâmetros</strong>';
    DialogService.show({
      name: 'install-dialog',
      type: 'warning',
      title: 'Instalação',
      text: `    
      ${textDialogPrefix}
      <br>Deseja realmente instalar/atualizar esse sistema?                      
        `,
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          outline: true,
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              DialogService.hide();
              this.executeInstall(row);
            },
          },
        },
      ],
    });
  }

  private async socketListener() {
    this.socket?.on('updaterInProgress', (result) => {
      const messages: any = {
        INSTALACAO_EM_CURSO: 'Instalação iniciada...',
        ATUALIZACAO_EM_CURSO: 'Atualização iniciada...',
        REMOCAO_EM_CURSO: 'Remoção iniciada...',
      };
      const {
        PRODFILIID,
      } = result.payload;
      let messageCode = '';
      switch (result.operation) {
        case 'INSTALL':
          messageCode = 'INSTALACAO_EM_CURSO';
          break;
        case 'UPDATE':
          messageCode = 'ATUALIZACAO_EM_CURSO';
          break;
        case 'REMOVE':
          messageCode = 'REMOCAO_EM_CURSO';
          break;
        default:
          messageCode = 'INSTALACAO_EM_CURSO';
          break;
      }
      if (ModalMultipleInstallController.opened) {
        ModalMultipleInstallController.updateGridRow(result.payload, { IDSTEVENTO: messageCode });
      } else {
        AlertHelper.showSuccess(messages[messageCode]);
      }
      DataSourceHelper.updateGridRow(this.getMainGrid(), 'PRODFILIID', PRODFILIID, { IDSTEVENTO: messageCode });
    });

    this.socket?.on('updaterFinish', (result) => {
      const {
        PRODFILIID,
        INSTALLVERSION: NRVERSAOATUAL,
        CURRENTVERSION,
      } = result.payload;

      if (result.error) {
        const IDSTEVENTO = CURRENTVERSION === null ? 'INSTALACAO_PENDENTE' : 'INSTALADO';
        if (ModalMultipleInstallController.opened) {
          ModalMultipleInstallController.updateGridRow(result.payload, { IDSTEVENTO }, true);
        } else {
          AlertHelper.showError('ERROR_RUNNING_LAST_PROCESS_CHECK_HISTORY_INFORMATION');
        }
        DataSourceHelper.updateGridRow(this.getMainGrid(), 'PRODFILIID', PRODFILIID, { IDSTEVENTO });
      } else {
        const IDSTEVENTO = result.operation === 'REMOVE' ? 'INSTALACAO_PENDENTE' : 'INSTALADO';
        if (ModalMultipleInstallController.opened) {
          ModalMultipleInstallController.updateGridRow(result.payload, { IDSTEVENTO, NRVERSAOATUAL });
        } else {
          AlertHelper.showSuccess('OPERATION_COMPLETED_SUCCESSFULLY');
        }
        const availableVersion: any = this.getAvailableVersion(PRODFILIID);
        if (availableVersion) {
          availableVersion.setValue(null);
        }
        DataSourceHelper.updateGridRow(this.getMainGrid(), 'PRODFILIID', PRODFILIID, { NRVERSAOATUAL, IDSTEVENTO });
      }
    });
    this.socket?.on('disconnect', () => {
      AlertHelper.showError('PAGE_WILL_RELOAD_TO_ESTABLISH_CONNECTION');
      setTimeout(() => {
        document.location.reload();
      }, 5000);
    });

    this.socket?.on('updaterConnected', async ({ UNITS, CONNECTED, WARNING = false }) => {
      const UNIT_CODES = UNITS.map((unit:any) => unit.CDFILIAL);
      DataSourceHelper.updateGridRow(this.getMainGrid(), 'CDFILIAL', UNIT_CODES, { CONNECTION_STATUS: CONNECTED, WARNING });
      ModalMultipleInstallController.updateGridRow({ CDFILIAL: UNIT_CODES }, { CONNECTION_STATUS: CONNECTED, WARNING });
    });

    this.socket?.on('updaterWarning', async ({ UNITS, WARNING = true }) => {
      const UNIT_CODES = UNITS.map((unit:any) => unit.CDFILIAL);
      DataSourceHelper.updateGridRow(this.getMainGrid(), 'CDFILIAL', UNIT_CODES, { WARNING });
    });

    this.socket?.on('updaterWithInvalidVersion', async (UNITS) => {
      const UNIT_CODES = UNITS.map((unit:any) => unit.CDFILIAL);
      DataSourceHelper.updateGridRow(this.getMainGrid(), 'CDFILIAL', UNIT_CODES, { CONNECTION_STATUS: false, WARNING: true });
    });

    this.socket?.on('detailStatusConnectionResult', async (INFO) => {
      await WarningController.openUpdaterDuplicateConnModal(INFO);
    });
  }

  public setAvailableVersionFilter({ component }: any) {
    const grid = this.getMainGrid();
    const [firstRow] = grid.datasource.data;
    const { ID } = firstRow;
    component.datasource.filter = {
      CDPRODUCTID: ID,
    };
  }

  public async beforeApplyFilter({ component, event }: any) {
    // Force to reload all available version column
    // component.datasource.updateData([]);
    event.preventDefault();
    const [productIdComponent] = component.getFilterInputs();
    const { value: PRODUCT_ID } = productIdComponent;
    this.updateGridData(PRODUCT_ID);
    ModalService.closeByName('updater-grid-filter-modal');
    component.toolbarSlot[0].text = `${this.defaultGridTitle} - ${this.filterRow.NMPRODUCT}`;
  }

  private getPage(dataset: any, page: number, limit: number) {
    const end: number = page * limit;
    const start = end - limit;
    const data = dataset.slice(end - limit, end);
    return {
      start,
      end: end > dataset.length ? dataset.length : end,
      data,
    };
  }

  public async updateGridData(PRODUCT_ID: any) {
    try {
      const grid = this.getMainGrid();
      LoadingService.show();
      const result = await Http.get('/updaterProdutosFilial', { params: { PRODUCT_ID } });
      (<any>grid.datasource).resultData = result.data.data;
      this.searchInGrid(grid.datasource.search);
      const { limit } = grid.datasource;
      const container: Container = grid.getChildInstance<Container>('page-numbers');
      const customPagination : CustomPagination = container.getChildInstance('custom-pagination');
      customPagination.length = Math.ceil(result.data.data.length / limit);
      this.onChangePage({ page: 1 });
    } catch (error) {
      AlertHelper.showError(error);
    } finally {
      LoadingService.hide();
    }
  }

  public changeProductId({ component }: any) {
    this.filterRow = component.selectValue;
  }

  public gridBeforeDestroy() {
    this.fullscreenModal?.destroy();
    this.socket?.removeAllListeners();
    this.socket?.disconnect();
    ModalService.getModal('updater-grid-filter-modal')?.destroy();
    ModalService.getModal('modal-log')?.destroy();
    WarningController.destroyModal('updater-duplicate-conn-modal');
    ModalMultipleInstallController.destroy();
  }

  public getConnectionColor({ row }: any) {
    const color = row.CONNECTION_STATUS ? 'green' : 'red';
    return row.WARNING ? '#FFCD01' : color;
  }

  public moreInfoConnection({ row }: any) {
    WarningController.moreInfoConnection(row, this.socket);
  }

  public changeSelectVersion({ value }: IEventParam<Select>) {
    const grid = this.getMainGrid();
    grid.datasource.currentRow.VERSION = value;
  }

  public beforeOpenReport(reportParams: any) {
    const grid = this.getMainGrid();
    reportParams.dataSet = (<any>grid.datasource).resultData;
    reportParams.dataSet = JSON.parse(JSON.stringify(reportParams.dataSet));
    reportParams.metaData.filter = [{ label: 'Produto', value: this.filterRow.NMPRODUCT }];
    const { componentProps: { datasource, dataValue, dataText } } = grid.getColumn('IDSTEVENTO');
    const events = datasource.data;
    const messageEvents = events.reduce((acc: any, cv: any) => {
      acc[cv[dataValue]] = cv[dataText];
      return acc;
    }, {});
    reportParams.dataSet.forEach((item: any) => {
      item.IDSTEVENTO = messageEvents[item.IDSTEVENTO] || item.IDSTEVENTO;
    });
    return reportParams;
  }

  public async openModalVersion({ row }: any) {
    const title = `Filial: ${row.CDFILIAL} - ${row.NMFILIAL}`;
    ModalVersion.setParams(row);
    ModalVersion.setTitle(title);
    ModalVersion.show();
  }

  public downloadZipLogs({ row }: any) {
    if (!this.verifyAndShowUnitStatus(row)) return;

    this.socket?.emit('downloadLog', { CDFILIAL: row.CDFILIAL, type: 'zip' });
    AlertHelper.showSuccess('DOWNLOAD_REQUEST_IN_PROGRESS');
    this.socket?.on('downloadLogResult', async ({ error, message = '', param }) => {
      if (error) {
        AlertHelper.showError(message);
      } else {
        AlertHelper.showSuccess('DOWNLOAD_IN_PROGRESS');
        const opened = window.open(param.downloadUrl);
        if (!opened) {
          AlertHelper.showAlert('POPUP_BLOCKER');
        }
      }
      this.socket?.removeAllListeners('downloadLogResult');
    });
  }
}
